@@ -94,6 +94,32 @@ create_indicators <- function(sens,n_sensitivity,elem,n_elem_before=0){
 }
 
 
+# Create an iterator based on sens of the current iteration within a scenario (DSA) --------------------------------------------------------
+#' Create an iterator based on sens of the current iteration within a scenario (DSA)
+#'
+#' @param sens current analysis iterator
+#' @param n_sensitivity total number of analyses to be run
+#'
+#' @return Integer iterator based on the number of sensitivity analyses being run and the total iterator
+#' @export
+#' 
+#' @details
+#' In a situation like a DSA, where two (low and high) scenarios are run, sens will go from 1 to n_sensitivity*2. However,
+#' this is not ideal as the parameter selector may depend on knowing the parameter order (i.e., 1, 2, 3...), which means
+#' resetting the counter back to 1 once sens reaches n_sensitivity (or any multiple of n_sensitivity) is needed.
+#'
+#' @examples
+#' sens_iterator(5,20)
+#' sens_iterator(25,20)
+
+sens_iterator <- function(sens,n_sensitivity){
+  
+  out <- sens - n_sensitivity*floor((sens-1)/n_sensitivity)
+  
+  return(out)
+}
+
+
 # Helper to draw PSA values --------------------------------------------------------
 #' Helper function to create a list with random draws or whenever a series of functions needs to be called. Can be implemented within `pick_val_v`.
 #'
@@ -193,6 +219,7 @@ pick_psa <- function(f,...){
 #' @param sens_iterator Current iterator number of the DSA/scenario being run, e.g., 5 if it corresponds to the 5th DSA parameter being changed
 #' @param distributions List with length equal to length of base where the distributions are stored
 #' @param covariances List with length equal to length of base where the variance/covariances are stored (only relevant if multivariate normal are being used)
+#' @param deploy_env Boolean, if TRUE will deploy all objects in the environment where the function is called for. Must be active if using add_item2 (and FALSE if using add_item)
 #'
 #' @return List used for the inputs
 #' @export
@@ -244,7 +271,8 @@ pick_val_v <- function(base,
                          indicator_sens_binary = TRUE,
                          sens_iterator = NULL,
                          distributions = NULL,
-                         covariances = NULL
+                         covariances = NULL,
+                         deploy_env = FALSE
 ){
   
   output <- list()
@@ -319,13 +347,17 @@ pick_val_v <- function(base,
     names(output) <- names_out
   }
   
-  return(as.list(output))
+  if(deploy_env){
+    list2env(as.list(output), parent.frame())
+  } else{
+    return(as.list(output))
+  }
 } 
 
 
 # Add item/parameter to list --------------------------------------------------------
 
-#' Defining parameters that may be used in model calculations
+#' Define parameters that may be used in model calculations (list)
 #'
 #' @param .data Existing data
 #' @param ... Items to define for the simulation
@@ -379,7 +411,62 @@ add_item <- function(.data=NULL,...){
   return(data_list)
 }
 
+# Add item/parameter (uses expressions) --------------------------------------------------------
 
+#' Define parameters that may be used in model calculations (uses expressions)
+#'
+#' @param .data Existing data
+#' @param input Items to define for the simulation as an expression (i.e., using {})
+#'
+#' @return A substituted expression to be evaluated by engine 
+#' @export
+#'
+#' @details
+#' The functions to add/modify events/inputs use lists. If chaining together add_item2, it will just append the expressions together in the order established.
+#'
+#' If using `pick_val_v`, note it should be used with the `deploy_env = TRUE` argument so that add_item2 process it correctly.
+#'
+#' @examples
+#' library(magrittr)
+#'
+#' add_item2(input = {fl.idfs <-  0})
+#' add_item2(input = {
+#'  util_idfs <- if(psa_bool){rnorm(1,0.8,0.2)} else{0.8}
+#'  util.mbc <- 0.6
+#'  cost_idfs <- 2500})
+#' common_inputs <- add_item2(input = {
+#' pick_val_v(
+#'   base      = l_statics[["base"]],
+#'   psa       = pick_psa(
+#'     l_statics[["function"]],
+#'     l_statics[["n"]],
+#'     l_statics[["a"]],
+#'     l_statics[["b"]]
+#'   ),
+#'   sens      = l_statics[[sens_name_used]],
+#'   psa_ind   = psa_bool,
+#'   sens_ind  = sensitivity_bool,
+#'   indicator = indicators_statics,
+#'   names_out = l_statics[["parameter_name"]],
+#'   deploy_env = TRUE #Note this option must be active if using it at add_item2
+#' )
+#' }
+#' )
+#'
+add_item2 <- function(.data=NULL,input){
+  
+  data_list <- .data
+  
+  list_item <- substitute(input)
+  
+  if (is.null(data_list)) {
+    data_list <- list_item
+  } else{
+    data_list <- as.call(c(substitute(`{`), as.list(data_list)[-1], as.list(list_item)[-1]))
+  }
+  
+  return(data_list)
+}
 
 # Add event to list of events ---------------------------------------------
 
@@ -411,13 +498,14 @@ new_event <- function(evt){
     stop("New event times are not all numeric, please review")
   }
   
-  input_list_arm <- parent.frame()$input_list_arm
-  
+  input_list_arm <- parent.frame()
+
   evtlist_temp <- list(cur_evtlist = c(input_list_arm$cur_evtlist,
                                   new_evt))
+
   if(input_list_arm$debug){ #only works correctly with create_if_null==TRUE, to be modified in later versions
     loc <- paste0("Analysis: ", input_list_arm$sens," ", input_list_arm$sens_name_used,
-                  "; Sim: ", input_list_arm$sim,
+                  "; Sim: ", input_list_arm$simulation,
                   "; Patient: ", input_list_arm$i,
                   "; Arm: ", input_list_arm$arm,
                   "; Event: ", input_list_arm$evt,
@@ -442,9 +530,6 @@ new_event <- function(evt){
   
   
   input_list_arm[["cur_evtlist"]] <- evtlist_temp$cur_evtlist
-  
-  list2env(input_list_arm["cur_evtlist"],envir = parent.frame())
-  assign("input_list_arm",input_list_arm, envir = parent.frame())
 
 }
 
@@ -477,7 +562,7 @@ new_event <- function(evt){
 #' add_reactevt(name_evt = "idfs",input = {modify_event(list("os"=5))})
 
 modify_event <- function(evt,create_if_null=TRUE){
-  input_list_arm <- parent.frame()$input_list_arm
+  input_list_arm <- parent.frame()
   
   evt_unlist <- unlist(evt)
   if(!is.numeric(evt_unlist)){
@@ -487,7 +572,7 @@ modify_event <- function(evt,create_if_null=TRUE){
   
   if(input_list_arm$debug){ #only works correctly with create_if_null==TRUE, to be modified in later versions
     loc <- paste0("Analysis: ", input_list_arm$sens," ", input_list_arm$sens_name_used,
-                  "; Sim: ", input_list_arm$sim,
+                  "; Sim: ", input_list_arm$simulation,
                   "; Patient: ", input_list_arm$i,
                   "; Arm: ", input_list_arm$arm,
                   "; Event: ", input_list_arm$evt,
@@ -528,10 +613,6 @@ modify_event <- function(evt,create_if_null=TRUE){
     input_list_arm[["cur_evtlist"]][names_evt] <- evt_unlist
   }
   
-   
-  list2env(input_list_arm["cur_evtlist"],envir = parent.frame())
-  assign("input_list_arm",input_list_arm, envir = parent.frame())
-  
 
 }
 
@@ -552,6 +633,10 @@ modify_event <- function(evt,create_if_null=TRUE){
 #' @details
 #' The functions to add/modify events/inputs use lists. Whenever several inputs/events are added or modified, it's recommended to group them within one function, as it reduces the computation cost.
 #' So rather than use two `modify_item` with a list of one element, it's better to group them into a single `modify_item` with a list of two elements.
+#' 
+#' Note that `modify_item` nor `modify_item_seq` can work on subelements (e.g.,
+#' `modify_item(list(obj$item = 5))` will not work as intended, for that is better
+#' to assign directly using the expression approach, so `obj$item <- 5`).
 #'
 #' Costs and utilities can be modified by using the construction `type_name_category`, where type is either "qaly" or "cost",
 #'  name is the name (e.g., "default") and category is the category used (e.g., "instant"), so one could pass `cost_default_instant` and modify the cost.
@@ -564,12 +649,13 @@ modify_event <- function(evt,create_if_null=TRUE){
 
 
 modify_item <- function(list_item){
-  input_list_arm <- parent.frame()$input_list_arm
+
+  input_list_arm <- parent.frame()
   
   if(input_list_arm$debug){ 
     
     loc <- paste0("Analysis: ", input_list_arm$sens," ", input_list_arm$sens_name_used,
-                  "; Sim: ", input_list_arm$sim,
+                  "; Sim: ", input_list_arm$simulation,
                   "; Patient: ", input_list_arm$i,
                   "; Arm: ", input_list_arm$arm,
                   "; Event: ", input_list_arm$evt,
@@ -593,14 +679,15 @@ modify_item <- function(list_item){
   }
   
   
-  input_list_arm[names(list_item)] <- lapply(list_item, unname)
+  list2env(lapply(list_item, unname), parent.frame())
   
   if(input_list_arm$accum_backwards){
-  input_list_arm[paste0(names(list_item),"_lastupdate",recycle0=TRUE)] <- 1
+    l_temp <- as.list(rep(1,length(list_item)))
+    names(l_temp) <- paste0(names(list_item),"_lastupdate",recycle0=TRUE)
+    list2env(l_temp, parent.frame())
   }
   
-  list2env(list_item,envir = parent.frame())
-  assign("input_list_arm",input_list_arm, envir = parent.frame())
+  
 }
 
 # Modify item in input list  evaluating sequentially -------------------------------------------------------------------------------------------------------------------------------
@@ -617,6 +704,10 @@ modify_item <- function(list_item){
 #' The functions to add/modify events/inputs use lists. Whenever several inputs/events are added or modified, it's recommended to group them within one function, as it reduces the computation cost.
 #' So rather than use two `modify_item` with a list of one element, it's better to group them into a single `modify_item` with a list of two elements.
 #'
+#' Note that `modify_item` nor `modify_item_seq` can work on subelements (e.g.,
+#' `modify_item_seq(list(obj$item = 5))` will not work as intended, for that is better
+#' to assign directly using the expression approach, so `obj$item <- 5`).
+#'
 #' Costs and utilities can be modified by using the construction `type_name_category`, where type is either "qaly" or "cost",
 #'  name is the name (e.g., "default") and category is the category used (e.g., "instant"), so one could pass `cost_default_instant` and modify the cost.
 #'  This will overwrite the value defined in the corresponding cost/utility section.
@@ -632,25 +723,25 @@ modify_item <- function(list_item){
 #'   })
 
 modify_item_seq <- function(...){
-  
-  input_list_arm <- parent.frame()$input_list_arm
+  input_list_arm <- parent.frame()
   input_list <- as.list(substitute(...))[-1]
   list_out <- list()
   
   if(input_list_arm$debug){ 
-    temp_dump <- input_list_arm[names(input_list)]
+    temp_dump <- mget(names(input_list),input_list_arm, ifnotfound = Inf)
   }
   
   for (inp in 1:length(input_list)) {
-    name_temp <- names(input_list[inp])
-    list_out[[ name_temp ]] <- eval(input_list[[inp]], input_list_arm)
-    input_list_arm[name_temp] <- list_out[name_temp]
+    temp_obj <- as.list(eval(input_list[[inp]], input_list_arm))
+    names(temp_obj) <- names(input_list[inp])
+    list2env(temp_obj, input_list_arm)
   }
+  list_out <- mget(names(input_list),input_list_arm)
   
   if(input_list_arm$debug){ 
     
     loc <- paste0("Analysis: ", input_list_arm$sens," ", input_list_arm$sens_name_used,
-                  "; Sim: ", input_list_arm$sim,
+                  "; Sim: ", input_list_arm$simulation,
                   "; Patient: ", input_list_arm$i,
                   "; Arm: ", input_list_arm$arm,
                   "; Event: ", input_list_arm$evt,
@@ -672,14 +763,17 @@ modify_item_seq <- function(...){
   }
   
   
-  list2env(list_out,envir = parent.frame())
   
+  
+  # list2env(list_out,envir = parent.frame())
+
   if(input_list_arm$accum_backwards){
-  input_list_arm[paste0(names(input_list),"_lastupdate",recycle0=TRUE)] <- 1
+    l_temp <- as.list(rep(1,length(input_list)))
+    names(l_temp) <- paste0(names(input_list),"_lastupdate",recycle0=TRUE)
+    list2env(l_temp, parent.frame())
   }
   
-  assign("input_list_arm",input_list_arm, envir = parent.frame())
-  
+
 }
 
 # Add_reactevt -------------------------------------------------------------------------------------------------------------------------------------------
@@ -726,8 +820,11 @@ add_reactevt <- function(.data=NULL,name_evt,input){
   }
   
   data_list <- .data
+  
+  
+  input_sub <- substitute(input)
 
-  evt_r <- list(list(react=substitute(input)))
+  evt_r <- list(list(react=input_sub))
   
   names(evt_r) <- paste(name_evt)
 
@@ -741,7 +838,71 @@ add_reactevt <- function(.data=NULL,name_evt,input){
   return(data_list)
 }
 
+# Random stream of uniform numbers -------------------------------------------------------------------------------------------------------------------------------------------
 
+#' Creates an environment (similar to R6 class) of random uniform numbers to be drawn from
+#'
+#' @param stream_size Length of the vector of random uniform values to initialize
+#'
+#' @return Self (environment) behaving similar to R6 class
+#'
+#' @export
+#'
+#' @details
+#' This function creates an environment object that behaves similar to an R6 class
+#' but offers more speed vs. an R6 class.
+#' 
+#' The object is always initialized (see example below) to a specific vector of
+#'  random uniform values. The user can then call the object with `obj$draw_number(n)`,
+#'  where n is an integer, and will return the first n elements of the created
+#'  vector of uniform values. It will automatically remove those indexes from the
+#'  vector, so the next time the user calls `obj$draw_n()` it will already consider
+#'  the next index.
+#'  
+#' The user can also access the latest elements drawn by accessing `obj$random_n` 
+#'  (useful for when performing a luck adjustment), the current stream still 
+#'  to be drawn using `obj$stream` and the original size (when created) using 
+#'  `obj$stream_size`.
+#'  
+#'  If performing luck adjustment, the user can always modify the random value
+#'  by using `obj$random_n <- luck_adj(...)` (only valid if used with the expression
+#'  approach, not with `modify_item`)
+#' 
+#' 
+#' @examples
+#' stream_1 <- random_stream(1000)
+#' number_1 <- stream_1$draw_n() #extract 1st index from the vector created
+#' identical(number_1,stream_1$random_n) #same value
+#' number_2 <- stream_1$draw_n() #gets 1st index (considers previous)
+#' identical(number_2,stream_1$random_n) #same value
+
+random_stream <- function(stream_size = 100) {
+  self <- environment()
+  
+  # Initialize the stream with random numbers
+  self$stream <- runif(stream_size)
+  self$stream_size <- stream_size
+  self$random_n <- numeric()
+  
+  # Function to generate a new stream of random numbers
+  self$generate_stream <- function(size = self$stream_size) {
+    self$stream <- runif(size)
+  }
+  # Function to draw a specified number of values from the stream and remove them
+  self$draw_n <- function(n=1) {
+    if (length(self$stream) < n) {
+      warning("Stream is smaller than the number of numbers drawn. Generating a new stream of the correct size.")
+      self$generate_stream(n)
+      self$stream_size <- n
+    }
+    seq_index <- seq_len(n)
+    self$random_n <- self$stream[seq_index]
+    self$stream <- self$stream[-seq_index]
+    return(self$random_n)
+  }
+  
+  return(self)
+}
 
 # Add drawing and initial event list -------------------------------------------------------------------------------------------------------------------------------------------
 
@@ -1171,12 +1332,24 @@ disc_cycle_v <- function(lcldr=0.035, lclprvtime=0, cyclelength,lclcurtime, lclv
 #'  and whether it's contained within a conditional statement
 #'
 #' @examples 
-#' a <- add_reactevt(name_evt="example",
-#'    input={
-#'       modify_item(list(w=5))
-#'    })
+#' evt_react_list2 <-
+#'   add_reactevt(name_evt = "sick",
+#'                input = {modify_item(list(a=1+5/3))
+#'                  assign("W", 5 + 3 / 6 )
+#'                  x[5] <- 18
+#'                  for(i in 1:5){
+#'                    assign(paste0("x_",i),5+3)
+#'                  }
+#'                  if(j == TRUE){
+#'                    y[["w"]] <- 612-31+3
+#'                  }#'                
+#'                  q_default <- 0
+#'                  c_default <- 0
+#'                  curtime   <- Inf
+#'                  d <- c <- k <- 67
+#'                })
 #'    
-#'  extract_from_reactions(a)
+#'  extract_from_reactions(evt_react_list2)
 #' 
 #'
 #' @export
@@ -1248,7 +1421,7 @@ ast_as_list <- function(ee) {
   }
 
 
-#' Extracts items and events by looking into modify_item, modify_item_seq, modify_event and new_event
+#' Extracts items and events by looking into assignments, modify_item, modify_item_seq, modify_event and new_event
 #'
 #' @param node Relevant node within the nested AST list
 #' @param conditional_flag Boolean whether the statement is contained within a conditional statement
@@ -1315,21 +1488,16 @@ extract_elements_from_list <- function(node, conditional_flag = FALSE) {
     stringsAsFactors = FALSE
   )
   
-  # Base case: check if the node is a relevant function
   if (is.list(node)) {
-    # Safely extract and convert node[[1]] to character
     func_name <- if (!is.null(node[[1]])) as.character(node[[1]]) else NULL
     
+    # Case 1: modify_* / new_event
     if (!is.null(func_name) && any(func_name %in% c("modify_item_seq", "modify_item", "modify_event", "new_event"))) {
-      # Determine type
       type <- if (any(func_name %in% c("modify_item_seq", "modify_item"))) "item" else "event"
-      
-      # Extract list elements
       list_expr <- node[[2]]
       if (is.list(list_expr)) {
-        definition <- unlist(extract_defs(node),recursive=TRUE)
-        definition <- definition[!names(definition)==""]
-        
+        definition <- unlist(extract_defs(node), recursive = TRUE)
+        definition <- definition[!names(definition) == ""]
         results_temp <- data.frame(
           name = names(definition),
           type = type,
@@ -1337,19 +1505,65 @@ extract_elements_from_list <- function(node, conditional_flag = FALSE) {
           definition = definition,
           stringsAsFactors = FALSE
         )
-        
-        results <- rbind(results,results_temp)
+        results <- rbind(results, results_temp)
       }
     }
     
-    # Check if the node is an `if` block
-    if (!is.null(func_name) && any(func_name == "if")) {
-      # Process the condition
+    # Case 2: assignment via <- or =
+    if (!is.null(func_name) && func_name %in% c("<-", "=")) {
+
+      lhs <- node[[2]]
+      rhs <- node[[3]]
+      # Detect and extract from complex LHS expressions
+      if (is.symbol(lhs)) {
+        target_name <- as.character(lhs)
+      } else if (is.list(lhs)) {
+        target_name <- clean_output(expr_from_list(lhs))
+      } else {
+        target_name <- NA
+      }
+
+      if (!is.na(target_name)) {
+        results <- rbind(results, data.frame(
+          name = target_name,
+          type = "item",
+          conditional_flag = conditional_flag,
+          definition = clean_output(expr_from_list(rhs)),
+          stringsAsFactors = FALSE
+        ))
+      }
+    }
+    
+    # Case 3: assignment via assign()
+    if (!is.null(func_name) && func_name == "assign") {
+      if (length(node) >= 3) {
+        varname <- node[[2]]
+        if (is.character(varname)) {
+          varname <- as.character(varname)
+        } else if (is.list(varname)) {
+          varname <- clean_output(expr_from_list(varname))
+        }
+        
+        value_expr <- node[[3]]
+        if (is.character(varname)) {
+          results <- rbind(results, data.frame(
+            name = varname,
+            type = "item",
+            conditional_flag = conditional_flag,
+            definition = clean_output(expr_from_list(value_expr)),
+            stringsAsFactors = FALSE
+          ))
+        }
+      }
+    }
+    
+    # Case 4: if statement → propagate conditional flag
+    if (!is.null(func_name) && func_name == "if") {
       conditional_flag <- TRUE
     }
   }
   
-  # Recursive case: iterate through list elements
+  # Recursively walk children
   if (is.list(node)) {
     for (child in node) {
       results <- rbind(results, extract_elements_from_list(child, conditional_flag))
@@ -1357,11 +1571,10 @@ extract_elements_from_list <- function(node, conditional_flag = FALSE) {
   }
   
   results <- results[!(is.na(results$name) | results$name == "" | is.na(results$definition)), ]
-  
-  
-  return(results)
+  results
 }
 
+
 #' Loop to extract an expression from a list
 #'
 #' @param lst sublist from the AST as list
@@ -1437,4 +1650,86 @@ extract_defs <- function(lst){
 
 
 
-
+#' Extract assignments from expression for debug mode
+#'
+#' @param expr Expression of type language 
+#'
+#' @return Vector of cleaned character objects that are assigned. It will ignore objects created dynamically (e.g., by pasting in a loop)
+#'
+#' @noRd
+#'  
+#' @examples
+#' 
+#'  expr <- substitute({
+#'    q_default <- if (fl.idfs==1) {
+#'      util.idfs.ontx * fl.idfs.ontx + (1-fl.idfs.ontx) * (1-fl.idfs.ontx) 
+#'    } else if (fl.idfs==0 & fl.mbcs==0) {
+#'      util.remission * fl.remission + fl.recurrence*util.recurrence
+#'    } else if (fl.mbcs==1) {
+#'      util.mbc.progression.mbc * fl.mbcs.progression.mbc + (1-fl.mbcs.progression.mbc)*util.mbc.pps
+#'    }
+#'    c_default <- cost.recurrence * fl.recurrence
+#'    fl.recurrence <- 1
+#'    fl.remission <- 0
+#'    fl.mbcs <- 1
+#'    fl.mbcs.progression.mbc <- 1 #ad-hoc for plot
+#'    if(a){c<-5}else{d<-6}
+#'    if(e){assign("t",6)}else{j<-6}
+#'    for(i in 1:10){
+#'      assign(paste0("a_",i),1)
+#'    }
+#'    a = 5
+#'    d <- b <- c <- 5
+#'  })
+#'  
+#'  extract_assignment_targets(expr)
+#' 
+#' 
+extract_assignment_targets <- function(expr) {
+  assigned <- character()
+  
+  walk_node <- function(node) {
+    if (is.call(node)) {
+      fname <- as.character(node[[1]])
+      
+      # Handle <- and = assignment
+      if (fname %in% c("<-", "=")) {
+        lhs <- node[[2]]
+        if (is.symbol(lhs)) {
+          assigned <<- c(assigned, as.character(lhs))
+        } else if (is.call(lhs) && lhs[[1]] == as.name("$")) {
+          # x$y <- ...  => get "x"
+          assigned <<- c(assigned, as.character(lhs[[2]]))
+        } else if (is.call(lhs) && lhs[[1]] == as.name("[[")) {
+          # x[["y"]] <- ... => get "x"
+          assigned <<- c(assigned, as.character(lhs[[2]]))
+        } else if (is.call(lhs) && lhs[[1]] == as.name("[")) {
+          assigned <<- c(assigned, as.character(lhs[[2]]))
+        }
+        walk_node(node[[3]])  # Recurse on RHS
+      }
+      
+      # Handle assign("var", value)
+      else if (fname == "assign" && length(node) >= 2) {
+        varname <- node[[2]]
+        if (is.character(varname)) {
+          assigned <<- c(assigned, varname)
+        } else if (is.symbol(varname)) {
+          assigned <<- c(assigned, as.character(varname))
+        }
+        # Recurse on value
+        if (length(node) >= 3) walk_node(node[[3]])
+      }
+      
+      # Recurse into all sub-calls
+      for (arg in as.list(node)[-1]) {
+        walk_node(arg)
+      }
+    } else if (is.expression(node) || is.call(node)) {
+      for (item in as.list(node)) walk_node(item)
+    }
+  }
+  
+  walk_node(expr)
+  unique(assigned)
+}
\ No newline at end of file
